from ussd.tests import UssdTestCase


class TestScreensUsingFilters(UssdTestCase.BaseUssdTestCase):

    validate_ussd = False

    def get_ussd_client(self):
        return self.ussd_client(
            generate_customer_journey=False,
            extra_payload={
                "customer_journey_conf": "sample_using_utility_filters.yml"
            }
        )

    def test_using_filters(self):
        client = self.get_ussd_client()
        # dial in
        response = client.send('')

        self.assertEqual(
            "Formatting numbers 1,000.00 and currency conversation "
            "Ksh 1,000.00\n", response)



from ussd.tests import UssdTestCase


class TestFunctionScreen(UssdTestCase.BaseUssdTestCase):
    validation_error_message = dict(
        get_name=dict(
            function=["Function ussd.tests.utils.show_name does not exist"]
        ),
        get_height=dict(
            function=["Module ussd.tests.util does not exist"]
        ),
        get_weight=dict(
            function=["Module name where function is located not given"]
        ),
        get_age=dict(
            next_screen=['This field is required.'],
            session_key=['This field is required.'],
            function=['This field is required.'],
        )
    )

    def test(self):
        client = self.ussd_client(phone_number='200')
        client.send('')  # dial in
        client.send('10')  # enter 10
        resp = client.send('1')

        expected_text = "The results was an {0} number " \
                        "which is {1}"

        self.assertEqual(
            expected_text.format('odd', 11),
            resp
        )

        client = self.ussd_client(phone_number='200')
        client.send('')  # dial in
        client.send('10')  # enter 10
        resp = client.send('2')

        expected_text = "The results was an {0} number " \
                        "which is {1}"

        self.assertEqual(
            expected_text.format('even', 12),
            resp
        )



from ussd.tests import UssdTestCase
from ussd import VERSION


class TestScreenUsing(UssdTestCase.BaseUssdTestCase):

    validate_ussd = False

    def get_ussd_client(self):
        return self.ussd_client(
            generate_customer_journey=False,
            extra_payload={
                "customer_journey_conf":
                    "teasting_using_built_in_functions.yml"
            }
        )

    def test(self):
        client = self.get_ussd_client()
        # dial in
        response = client.send('1')

        self.assertEqual(
            "You are using this version v{version}".format(version=VERSION),
            response
        )



from ussd.tests import UssdTestCase


class TestMultiLanguage(UssdTestCase.BaseUssdTestCase):
    validate_ussd = False

    def get_ussd_client(self):
        return self.ussd_client(
            generate_customer_journey=False,
            extra_payload={
                "customer_journey_conf": "valid_multi_language_support_from_session_data_conf.yml"
            }
        )

    def test_multilanguage_support(self):

        ussd_client = self.ussd_client(language='sw')


        # Dial in
        response = ussd_client.send('')

        self.assertEqual(
            "Bienvenue\n1. Bienvenue\n",
            response
        )

        response = ussd_client.send('1')

        self.assertEqual(
            "Pour compléter, saisissez une valeur\n",
            response
        )

        response = ussd_client.send('1')

        self.assertEqual(
            "Au revoir",
            response
        )


from ussd.core import UssdHandlerAbstract
from ussd.screens.serializers import NextUssdScreenSerializer
from rest_framework import serializers
import importlib


class FunctionScreenSerializer(NextUssdScreenSerializer):
    """
    Fields used to create this screen:

    1. function
        This is the function that will be called at this screen.
    2. session_key
        Once your function has been called the output of your function
        will be saved in ussd session using session_key
    3. next_screen
        Once your function has been called this it goes to the
        screen specified in next_screen
    """
    session_key = serializers.CharField()
    function = serializers.CharField()

    @staticmethod
    def validate_function(value):
        split_path = value.split('.')
        if len(split_path) <= 1:
            raise serializers.ValidationError(
                "Module name where function is located not given"
            )
        function_name = split_path[-1]
        module_name = '.'.join(value.split('.')[:-1])
        try:
            module = importlib.import_module(module_name)
        except ImportError:
            raise serializers.ValidationError(
                "Module {0} does not exist".format(module_name)
            )

        if not hasattr(module, function_name):
            raise serializers.ValidationError(
                "Function {0} does not exist".format(value)
            )


class FunctionScreen(UssdHandlerAbstract):
    """
    This screen is invisible to the user. Its used to if you want to call a function you
    have implemented.

    Its like a complement of http screen. In http screen you make a request to an
    external service to perform some logic.

    This screen on the contrary if the logic that you want to be executed is within
    your application you use this screen to execute it.

    Your function will be called with UssdRequest object.
    And it should return a dictionary that will be saved in ussd session

    Below is the UssdRequest that will be used.
        .. autoclass:: ussd.core.UssdRequest

    Screen specification
        .. autoclass:: ussd.screens.function_screen.FunctionScreenSerializer

    Examples of function screens:
        .. literalinclude:: .././ussd/tests/sample_screen_definition/valid_function_screen_conf.yml
    """
    screen_type = "function_screen"
    serializer = FunctionScreenSerializer

    def handle(self):
        split_path = self.screen_content['function'].split('.')
        function_name = split_path[-1]
        module_name = '.'.join(split_path[:-1])
        module = importlib.import_module(module_name)

        self.ussd_request.session[
            self.screen_content['session_key']
        ] = getattr(module, function_name)(self.ussd_request)

        return self.route_options()



from django.conf.urls import url
from ussd.views import journey_visual,journey_visual_data

urlpatterns = [
    url(r'ussd-airflow/journey_visual$',journey_visual,name="journey_visual"),
    url(r'ussd-airflow/journey_visual_data$',journey_visual_data,name='journey_visual_data')
]



from ussd.core import register_filter


@register_filter
def format_number(value):
    """
    formats the number into the commas and 2 decimal places
    :param value:
    :return:
    """
    val = '0' if value == '' else value
    return "{:,.2f}".format(int(val))


@register_filter
def format_currency(value, currency):
    """
    formats the number into the commas and 2 decimal places
    :param value:
    :param currency:
    :return:
    """
    val = '0' if value == '' else value
    curr = 'KSH' if currency == '' else currency

    return "{currency} {value:,.2f}".format(currency=curr, value=int(val))



import importlib
import yaml
from datetime import datetime

date_format = "%Y-%m-%d %H:%M:%S.%f"


def str_to_class(import_path):
    module_name, class_name = import_path.rsplit(".", 1)
    try:
        module_ = importlib.import_module(module_name)
        try:
            class_ = getattr(module_, class_name)
        except AttributeError:
            raise Exception('Class does not exist')
    except ImportError:
        raise Exception('Module does not exist')
    return class_


def datetime_to_string(date_obj: datetime):
    return date_obj.strftime(date_format)


def string_to_datetime(date_str_obj: str):
    return datetime.strptime(date_str_obj, date_format)


class YamlToGo:
    def __init__(self,file):
        file = open(file)
        self.yaml = yaml.load(file)
        file.close()

        self.count = 0

    def get_model_data(self):
        data = {}
        links = []
        for index,value in enumerate(self.yaml):
            data[value] = self.convert_screen(value,index)

        for index,value in enumerate(self.yaml):
            links.extend(self.get_links(value,data))

        dataArray = []
        dataArray.append(data.pop('initial_screen'))
        dataArray.extend(list(data.values()))

        return {'data':dataArray,'links':links}

    def get_text(self,text):
        if type(text) is str:
            return text
        else:
            return text['en']

    def convert_screen(self,screen,key):
        _data = {}
        name = screen
        screen = self.yaml[screen]
        type = screen['type']
        _data['key'] = key
        _data['title'] = name

        if type == 'initial_screen':
            _data['items'] = [
                {'index':1,'portName':'out1','text':'Initial Screen'}
            ]
        elif type == 'input_screen':
            _data['items'] = [
                {'index':'1','portName':'out1','text':self.get_text(screen['text'])}
            ]
        elif type.endswith('http_screen'):
            _data['items'] = [
                {'index':'1','portName':'out1','text':'Http Request Screen'}
            ]

        elif type == 'menu_screen':
            _data['items'] = []
            for index,value in enumerate(screen['options']):
                _data['items'].append({
                    'index' : index+1,
                    'text' : self.get_text(value['text']),
                    'portName': 'out' + str(index)
                })

        elif type == 'router_screen':
            _data['items'] = []

            for index,value in enumerate(screen['router_options']):
                _data['items'].append({
                    'index' : index+1,
                    'text' : value['expression'],
                    'portName': 'out' + str(index)
                })

            if 'default_next_screen' in screen:
                _data['items'].append({
                    'index' : len(screen['router_options'])+1,
                    'text' : 'Default Route',
                    'portName':'outDefault'
                })


        elif type == 'quit_screen':
            _data['items'] = [
                {'index':1,'portName':'out1','text':self.get_text(screen['text'])}
            ]

        return _data

    def get_links(self,screen,data):
        _data = []
        name = screen
        screen = self.yaml[screen]
        type = screen['type']

        if type == 'initial_screen':
            _data = [
                {
                    'from' : data[name]['key'],
                    'to' : data[screen['next_screen']]['key'],
                    'fromPort':'out1'
                }
            ]
        elif type == 'input_screen':
            _data = [
                {
                    'from': data[name]['key'],
                    'to': data[screen['next_screen']]['key'],
                    'fromPort': 'out1'
                }
            ]
        elif type.endswith('http_screen'):
            _data = [
                {
                    'from': data[name]['key'],
                    'to': data[screen['next_screen']]['key'],
                    'fromPort': 'out1'
                }
            ]

        elif type == 'menu_screen':
            for index, value in enumerate(screen['options']):
                _data.append({
                    'from': data[name]['key'],
                    'to': data[value['next_screen']]['key'],
                    'fromPort': 'out' + str(index)
                })

        elif type == 'router_screen':
            if 'default_next_screen' in screen:
                _data.append({
                    'from' : data[name]['key'],
                    'to' : data[screen['default_next_screen']]['key'],
                    'fromPort' : 'outDefault'
                })
            for index, value in enumerate(screen['router_options']):
                _data.append({
                    'from': data[name]['key'],
                    'to': data[value['next_screen']]['key'],
                    'fromPort': 'out' + str(index)
                })


        elif type == 'quit_screen':
            pass

        return _data
